
#include <pbs_config.h>   /* the master config generated by configure */

#include <stdio.h>
#include <stdlib.h>
#include <netdb.h>
#include <string.h>

#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>

extern int h_errno;

void usage(name)
	char *name;
{
		fprintf(stderr, "Usage: %s [-v] hostname\n", name);
		fprintf(stderr, "\t -v turns on verbose output\n");
}

main(argc, argv, env)
	int argc;
	char *argv[];
	char *env[];
{
	int i;
	struct hostent *host;
	struct hostent *hosta;
	struct in_addr *ina;
	int		naddr;
	int		vflag = 0;
	void prt_herrno();
	extern int optind;

	while ((i = getopt(argc, argv, "v")) != EOF) {
		switch (i) {
		    case 'v':	vflag = 1;
				break;
		    default:
				usage(argv[0]);
				return 1;
		}
	}

	if (optind != argc-1) {
		usage(argv[0]);
		return 1;
	}

	h_errno = 0;

	i = 0;
	while (env[i]) {
		if (!strncmp(env[i], "LOCALDOMAIN", 11)) {
			printf("%s\n", env[i]);
			env[i] = "";
			break;
		}
		++i;
	}

	host = gethostbyname(argv[optind]);
	if (host) {
		printf("primary name:  %s", host->h_name);
		if (vflag)
			 printf(" (from gethostbyname())");
		printf("\n");
		if (host->h_aliases && *host->h_aliases) {
			for (i=0; host->h_aliases[i]; ++i)
				printf("aliases:           %s\n",
					host->h_aliases[i]);
		} else if (vflag) {
			printf("aliases:            -none-\n");
		}

		if (vflag)
			printf("     address length:  %d bytes\n", host->h_length);

		/* need to save address because they will be over writen on */
		/* next call to gethostby*()				    */

		naddr = 0;
		for (i=0; host->h_addr_list[i]; ++i) {
			++naddr;
		}
		ina = (struct in_addr *)malloc(sizeof (struct in_addr) * naddr);
		
		for (i=0; i<naddr; ++i) {
			(void)memcpy((char *)(ina+i), host->h_addr_list[i],
						   host->h_length);
		}
		for (i=0; i<naddr; ++i) {
			printf("     address:      %15.15s  ", inet_ntoa(*(ina+i)));
			if (vflag)
				printf(" (%u dec)  ", (int)(ina+i)->s_addr);
		
			h_errno = 0;
			hosta = gethostbyaddr((char *)(ina+i), host->h_length,
							    host->h_addrtype);
			if (hosta) {
				printf("name:  %s", host->h_name);
			} else {
				printf("name:  -null-");
				prt_herrno();
			}
			printf("\n");
		}

	} else {
		printf("no name entry found for %s\n", argv[optind]);
		prt_herrno();
	}
	return 0;
}

void prt_herrno()
{
	char *txt;

	switch (h_errno) {
		case 0:
			return;

		case HOST_NOT_FOUND:
			txt = "Answer Host Not Found";
			break;

		case TRY_AGAIN:
			txt = "Try Again";
			break;

		case NO_RECOVERY:
			txt = "No Recovery";
			break;

		case NO_DATA:
			txt = "No Data";
			break;

		default:	
			txt = "unknown error";
			break;
	}
	printf(" ** h_errno is %d %s\n", h_errno, txt);
}
